import * as React from "react";
import ReactMarkdown from "react-markdown";
import { InlineCode } from "./InlineCode";
import {
  Link,
  Popover,
  PopoverArrow,
  PopoverBody,
  PopoverCloseButton,
  PopoverContent,
  PopoverHeader,
  PopoverTrigger,
  Portal,
  Stack,
  useColorModeValue,
} from "@chakra-ui/react";
import { MarkdownCodeBlock } from "@apollo/chakra-helpers";

type KeyTermProps = {
  definition: string;
  term: string;
};

export const KeyTerm = ({ definition, term }: KeyTermProps): JSX.Element => {
  const bg = useColorModeValue("gray.100", "gray.700");
  const borderColor = useColorModeValue("gray.300", "gray.600");

  const shadow = useColorModeValue(
    "rgba(0, 0, 0, 0.14)",
    "rgba(20, 20, 80, 1)"
  );

  return (
    <Popover
      onOpen={() => {
        window.gtag?.("event", "apollopedia_open", {
          term,
        });
      }}
    >
      <PopoverTrigger>
        <Link
          as="span"
          sx={{
            textDecoration: "underline dotted 1.5px",
            textDecorationColor: "gray.300",
            textUnderlineOffset: "0.2em",
            _dark: {
              textDecorationColor: "blue.400",
            },
          }}
          _hover={{ cursor: "help" }}
        >
          {term}
        </Link>
      </PopoverTrigger>
      <Portal>
        <PopoverContent
          borderColor={borderColor}
          bg={bg}
          minW={"sm"}
          w={
            definition.length > 500
              ? "xl"
              : definition.length > 300
              ? "md"
              : "sm"
          }
          _focus={{ boxShadow: `2px 2px 8px 0px ${shadow}` }}
        >
          <PopoverArrow bg={bg} />
          <PopoverCloseButton />
          <PopoverHeader borderColor={borderColor} fontWeight="bold">
            {term}
          </PopoverHeader>
          <PopoverBody fontSize="0.9em" lineHeight="1.3em" px="4" pb="4">
            <Stack spacing="2">
              <ReactMarkdown
                components={{
                  code: InlineCode,
                  pre: (props) => <MarkdownCodeBlock {...props} disableCopy />,
                }}
              >
                {definition}
              </ReactMarkdown>
            </Stack>
          </PopoverBody>
        </PopoverContent>
      </Portal>
    </Popover>
  );
};
